@@ -1,18 +1,18 @@
 #!/usr/bin/env -S deno run --allow-write=. --lock=./tools/deno.lock.json
-// Copyright 2018-2025 the Deno authors. MIT license.
+// Copyright 2018-2026 the Deno authors. MIT license.
 import { stringify } from "jsr:@std/yaml@^0.221/stringify";
 
 // Bump this number when you want to purge the cache.
 // Note: the tools/release/01_bump_crate_versions.ts script will update this version
 // automatically via regex, so ensure that this line maintains this format.
-const cacheVersion = 79;
+const cacheVersion = 91;
 
 const ubuntuX86Runner = "ubuntu-24.04";
 const ubuntuX86XlRunner = "ghcr.io/cirruslabs/ubuntu-runner-amd64:24.04";
-const ubuntuARMRunner = "ubicloud-standard-16-arm";
+const ubuntuARMRunner = "ghcr.io/cirruslabs/ubuntu-runner-arm64:24.04-plus";
 const windowsX86Runner = "windows-2022";
 const windowsX86XlRunner = "windows-2022-xl";
-const macosX86Runner = "macos-13";
+const macosX86Runner = "macos-15-intel";
 const macosArmRunner = "macos-14";
 const selfHostedMacosArmRunner = "ghcr.io/cirruslabs/macos-runner:sonoma";
 
@@ -76,7 +76,7 @@ const prCachePath = [
 ].join("\n");
 
 // Note that you may need to add more version to the `apt-get remove` line below if you change this
-const llvmVersion = 20;
+const llvmVersion = 21;
 const installPkgsCommand =
   `sudo apt-get install -y --no-install-recommends clang-${llvmVersion} lld-${llvmVersion} clang-tools-${llvmVersion} clang-format-${llvmVersion} clang-tidy-${llvmVersion}`;
 const sysRootStep = {
@@ -164,7 +164,7 @@ CFLAGS=$CFLAGS
 
 const installBenchTools = "./tools/install_prebuilt.js wrk hyperfine";
 
-const cloneRepoStep = [{
+const cloneRepoSteps = [{
   name: "Configure git",
   run: [
     "git config --global core.symlinks true",
@@ -215,6 +215,49 @@ const installDenoStep = {
   uses: "denoland/setup-deno@v2",
   with: { "deno-version": "v2.x" },
 };
+const cacheCargoHomeStep = {
+  name: "Cache Cargo home",
+  uses: "cirruslabs/cache@v4",
+  with: {
+    // See https://doc.rust-lang.org/cargo/guide/cargo-home.html#caching-the-cargo-home-in-ci
+    // Note that with the new sparse registry format, we no longer have to cache a `.git` dir
+    path: [
+      "~/.cargo/.crates.toml",
+      "~/.cargo/.crates2.json",
+      "~/.cargo/bin",
+      "~/.cargo/registry/index",
+      "~/.cargo/registry/cache",
+      "~/.cargo/git/db",
+    ].join("\n"),
+    key:
+      `${cacheVersion}-cargo-home-\${{ matrix.os }}-\${{ matrix.arch }}-\${{ hashFiles('Cargo.lock') }}`,
+    // We will try to restore from the closest cargo-home we can find
+    "restore-keys":
+      `${cacheVersion}-cargo-home-\${{ matrix.os }}-\${{ matrix.arch }}-`,
+  },
+};
+const restoreCachePrStep = {
+  // Restore cache from the latest 'main' branch build.
+  name: "Restore cache build output (PR)",
+  uses: "actions/cache/restore@v4",
+  if:
+    "github.ref != 'refs/heads/main' && !startsWith(github.ref, 'refs/tags/')",
+  with: {
+    path: prCachePath,
+    key: "never_saved",
+    "restore-keys": prCacheKeyPrefix,
+  },
+};
+const saveCacheMainStep = {
+  // In main branch, always create a fresh cache
+  name: "Save cache build output (main)",
+  uses: "actions/cache/save@v4",
+  if: "matrix.job == 'test' && github.ref == 'refs/heads/main'",
+  with: {
+    path: prCachePath,
+    key: prCacheKey,
+  },
+};
 
 const authenticateWithGoogleCloud = {
   name: "Authenticate with Google Cloud",
@@ -256,9 +299,16 @@ function withCondition(
   step: Record<string, unknown>,
   condition: string,
 ): Record<string, unknown> {
+  function maybeParens(condition: string) {
+    if (condition.includes("&&") || condition.includes("||")) {
+      return `(${condition})`;
+    } else {
+      return condition;
+    }
+  }
   return {
     ...step,
-    if: "if" in step ? `${condition} && (${step.if})` : condition,
+    if: "if" in step ? `${maybeParens(condition)} && (${step.if})` : condition,
   };
 }
 
@@ -351,7 +401,7 @@ const ci = {
         skip_build: "${{ steps.check.outputs.skip_build }}",
       },
       steps: onlyIfDraftPr([
-        ...cloneRepoStep,
+        ...cloneRepoSteps,
         {
           id: "check",
           if: "!contains(github.event.pull_request.labels.*.name, 'ci-draft')",
@@ -432,10 +482,6 @@ const ci = {
             job: "test",
             profile: "debug",
             use_sysroot: true,
-          }, {
-            ...Runners.linuxX86,
-            job: "lint",
-            profile: "debug",
           }, {
             ...Runners.linuxArm,
             job: "test",
@@ -446,14 +492,6 @@ const ci = {
             profile: "release",
             use_sysroot: true,
             skip_pr: true,
-          }, {
-            ...Runners.macosX86,
-            job: "lint",
-            profile: "debug",
-          }, {
-            ...Runners.windowsX86,
-            job: "lint",
-            profile: "debug",
           }]),
         },
         // Always run main branch builds to completion. This allows the cache to
@@ -471,7 +509,7 @@ const ci = {
         RUST_LIB_BACKTRACE: 0,
       },
       steps: skipJobsIfPrAndMarkedSkip([
-        ...cloneRepoStep,
+        ...cloneRepoSteps,
         submoduleStep("./tests/util/std"),
         {
           ...submoduleStep("./tests/wpt/suite"),
@@ -500,37 +538,21 @@ const ci = {
             "    -czvf target/release/deno_src.tar.gz -C .. deno",
           ].join("\n"),
         },
-        {
-          name: "Cache Cargo home",
-          uses: "cirruslabs/cache@v4",
-          with: {
-            // See https://doc.rust-lang.org/cargo/guide/cargo-home.html#caching-the-cargo-home-in-ci
-            // Note that with the new sparse registry format, we no longer have to cache a `.git` dir
-            path: [
-              "~/.cargo/.crates.toml",
-              "~/.cargo/.crates2.json",
-              "~/.cargo/bin",
-              "~/.cargo/registry/index",
-              "~/.cargo/registry/cache",
-              "~/.cargo/git/db",
-            ].join("\n"),
-            key:
-              `${cacheVersion}-cargo-home-\${{ matrix.os }}-\${{ matrix.arch }}-\${{ hashFiles('Cargo.lock') }}`,
-            // We will try to restore from the closest cargo-home we can find
-            "restore-keys":
-              `${cacheVersion}-cargo-home-\${{ matrix.os }}-\${{ matrix.arch }}-`,
-          },
-        },
+        cacheCargoHomeStep,
         installRustStep,
         {
-          if:
-            "matrix.job == 'lint' || matrix.job == 'test' || matrix.job == 'bench'",
+          if: "matrix.os == 'linux' && matrix.arch == 'aarch64'",
+          name: "Load 'vsock_loopback; kernel module",
+          run: "sudo modprobe vsock_loopback",
+        },
+        {
+          if: "matrix.job == 'test' || matrix.job == 'bench'",
           ...installDenoStep,
         },
         ...installPythonSteps.map((s) =>
           withCondition(
             s,
-            "matrix.job != 'lint' && (matrix.os != 'linux' || matrix.arch != 'aarch64')",
+            "matrix.os != 'linux' || matrix.arch != 'aarch64'",
           )
         ),
         {
@@ -653,18 +675,7 @@ const ci = {
             installBenchTools,
           ].join("\n"),
         },
-        {
-          // Restore cache from the latest 'main' branch build.
-          name: "Restore cache build output (PR)",
-          uses: "actions/cache/restore@v4",
-          if:
-            "github.ref != 'refs/heads/main' && !startsWith(github.ref, 'refs/tags/')",
-          with: {
-            path: prCachePath,
-            key: "never_saved",
-            "restore-keys": prCacheKeyPrefix,
-          },
-        },
+        restoreCachePrStep,
         {
           name: "Apply and update mtime cache",
           if: "!startsWith(github.ref, 'refs/tags/')",
@@ -681,34 +692,6 @@ const ci = {
             key: "playwright-${{ runner.os }}-${{ runner.arch }}",
           },
         },
-        {
-          name: "test_format.js",
-          if: "matrix.job == 'lint' && matrix.os == 'linux'",
-          run:
-            "deno run --allow-write --allow-read --allow-run --allow-net ./tools/format.js --check",
-        },
-        {
-          name: "lint.js",
-          if: "matrix.job == 'lint'",
-          env: {
-            GITHUB_TOKEN: "${{ secrets.GITHUB_TOKEN }}",
-          },
-          run:
-            "deno run --allow-write --allow-read --allow-run --allow-net --allow-env ./tools/lint.js",
-        },
-        {
-          name: "jsdoc_checker.js",
-          if: "matrix.job == 'lint'",
-          run:
-            "deno run --allow-read --allow-env --allow-sys ./tools/jsdoc_checker.js",
-        },
-        {
-          name: "Check tracing build",
-          if:
-            "matrix.job == 'test' && matrix.profile == 'debug' && matrix.os == 'linux' && matrix.arch == 'x86_64'",
-          run: "cargo check -p deno --features=lsp-tracing",
-          env: { CARGO_PROFILE_DEV_DEBUG: 0 },
-        },
         {
           name: "Build debug",
           if: "matrix.job == 'test' && matrix.profile == 'debug'",
@@ -867,7 +850,7 @@ const ci = {
             "github.repository == 'denoland/deno' &&",
             "(github.ref == 'refs/heads/main' || startsWith(github.ref, 'refs/tags/'))",
           ].join("\n"),
-          uses: "azure/trusted-signing-action@v0",
+          uses: "Azure/artifact-signing-action@v0",
           with: {
             "endpoint": "https://eus.codesigning.azure.net/",
             "trusted-signing-account-name": "deno-cli-code-signing",
@@ -999,6 +982,29 @@ const ci = {
             "fi",
           ].join("\n"),
         },
+        {
+          name: "Combine test results",
+          if: [
+            "always() &&",
+            "matrix.job == 'test' &&",
+            "!startsWith(github.ref, 'refs/tags/')",
+          ].join("\n"),
+          run: "deno run -RWN ./tools/combine_test_results.js",
+        },
+        {
+          name: "Upload test results",
+          uses: "actions/upload-artifact@v4",
+          if: [
+            "always() &&",
+            "matrix.job == 'test' &&",
+            "!startsWith(github.ref, 'refs/tags/')",
+          ].join("\n"),
+          with: {
+            name:
+              "test-results-${{ matrix.os }}-${{ matrix.arch }}-${{ matrix.profile }}.json",
+            path: "target/test_results.json",
+          },
+        },
         {
           name: "Configure hosts file for WPT",
           if: "matrix.wpt",
@@ -1075,7 +1081,7 @@ const ci = {
           run: "cargo bench --locked",
         },
         {
-          name: "Post Benchmarks",
+          name: "Post benchmarks",
           if: [
             "matrix.job == 'bench' &&",
             "github.repository == 'denoland/deno' &&",
@@ -1100,7 +1106,7 @@ const ci = {
         {
           name: "Build product size info",
           if:
-            "matrix.job != 'lint' && matrix.profile != 'debug' && github.repository == 'denoland/deno' && (github.ref == 'refs/heads/main' || startsWith(github.ref, 'refs/tags/'))",
+            "matrix.profile != 'debug' && github.repository == 'denoland/deno' && (github.ref == 'refs/heads/main' || startsWith(github.ref, 'refs/tags/'))",
           run: [
             'du -hd1 "./target/${{ matrix.profile }}"',
             'du -ha  "./target/${{ matrix.profile }}/deno"',
@@ -1202,19 +1208,66 @@ const ci = {
             draft: true,
           },
         },
-        {
-          // In main branch, always create a fresh cache
-          name: "Save cache build output (main)",
-          uses: "actions/cache/save@v4",
-          if:
-            "(matrix.job == 'test' || matrix.job == 'lint') && github.ref == 'refs/heads/main'",
-          with: {
-            path: prCachePath,
-            key: prCacheKey,
-          },
-        },
+        saveCacheMainStep,
       ]),
     },
+    lint: {
+      name: "lint ${{ matrix.profile }} ${{ matrix.os }}-${{ matrix.arch }}",
+      needs: ["pre_build"],
+      if: "${{ needs.pre_build.outputs.skip_build != 'true' }}",
+      "runs-on": "${{ matrix.runner }}",
+      "timeout-minutes": 30,
+      defaults: {
+        run: {
+          shell: "bash",
+        },
+      },
+      strategy: {
+        matrix: {
+          include: [{
+            ...Runners.linuxX86,
+            profile: "debug",
+            job: "lint",
+          }, {
+            ...Runners.macosX86,
+            profile: "debug",
+            job: "lint",
+          }, {
+            ...Runners.windowsX86,
+            profile: "debug",
+            job: "lint",
+          }],
+        },
+      },
+      steps: [
+        ...cloneRepoSteps,
+        submoduleStep("./tests/util/std"),
+        cacheCargoHomeStep,
+        installRustStep,
+        installDenoStep,
+        restoreCachePrStep,
+        {
+          name: "test_format.js",
+          if: "matrix.os == 'linux'",
+          run:
+            "deno run --allow-write --allow-read --allow-run --allow-net ./tools/format.js --check",
+        },
+        {
+          name: "lint.js",
+          env: {
+            GITHUB_TOKEN: "${{ secrets.GITHUB_TOKEN }}",
+          },
+          run:
+            "deno run --allow-write --allow-read --allow-run --allow-net --allow-env ./tools/lint.js",
+        },
+        {
+          name: "jsdoc_checker.js",
+          run:
+            "deno run --allow-read --allow-env --allow-sys ./tools/jsdoc_checker.js",
+        },
+        saveCacheMainStep,
+      ],
+    },
     libs: {
       name: "build libs",
       needs: ["pre_build"],
@@ -1222,7 +1275,7 @@ const ci = {
       "runs-on": ubuntuX86Runner,
       "timeout-minutes": 30,
       steps: skipJobsIfPrAndMarkedSkip([
-        ...cloneRepoStep,
+        ...cloneRepoSteps,
         installRustStep,
         {
           name: "Install wasm target",
@@ -1247,6 +1300,7 @@ const ci = {
             "cargo check --no-default-features --features package_json -p deno_config",
             "cargo check --no-default-features --features workspace --features sync -p deno_config",
             "cargo check --target wasm32-unknown-unknown --all-features -p deno_config",
+            "cargo check -p deno --features=lsp-tracing",
           ].join("\n"),
         },
       ]),
