@@ -1,4 +1,4 @@
-// Copyright 2018-2025 the Deno authors. MIT license.
+// Copyright 2018-2026 the Deno authors. MIT license.
 
 // TODO(petamoriken): enable prefer-primordials for node polyfills
 // deno-lint-ignore-file prefer-primordials
@@ -10,6 +10,7 @@ import {
 import { kMaxUserId } from "ext:deno_node/internal/fs/utils.mjs";
 import { validateInteger } from "ext:deno_node/internal/validators.mjs";
 import { op_fs_fchown_async, op_fs_fchown_sync } from "ext:core/ops";
+import { promisify } from "ext:deno_node/internal/util.mjs";
 
 /**
  * Changes the owner and group of a file.
@@ -44,3 +45,9 @@ export function fchownSync(
 
   op_fs_fchown_sync(fd, uid, gid);
 }
+
+export const fchownPromise = promisify(fchown) as (
+  fd: number,
+  uid: number,
+  gid: number,
+) => Promise<void>;
