@@ -1,4 +1,4 @@
-// Copyright 2018-2025 the Deno authors. MIT license.
+// Copyright 2018-2026 the Deno authors. MIT license.
 
 import { Buffer } from "node:buffer";
 import { assert, libSuffix } from "./common.js";
@@ -10,7 +10,7 @@ Deno.test("ctr initialization (napi_module_register)", {
   ignore: Deno.build.os == "windows",
 }, function () {
   const path = new URL(`./module.${libSuffix}`, import.meta.url).pathname;
-  const obj = ops.op_napi_open(path, {}, Buffer, reportError);
+  const obj = ops.op_napi_open(path, {}, Buffer.from, reportError);
   assert(obj != null);
   assert(typeof obj === "object");
 });
@@ -19,7 +19,7 @@ Deno.test("ctr initialization by multiple threads (napi_module_register)", {
   ignore: Deno.build.os == "windows",
 }, async function () {
   const path = new URL(`./module.${libSuffix}`, import.meta.url).pathname;
-  const obj = ops.op_napi_open(path, {}, Buffer, reportError);
+  const obj = ops.op_napi_open(path, {}, Buffer.from, reportError);
   const common = import.meta.resolve("./common.js");
   assert(obj != null);
   assert(typeof obj === "object");
@@ -29,9 +29,9 @@ Deno.test("ctr initialization by multiple threads (napi_module_register)", {
     import { Buffer } from "node:buffer";
     import { parentPort } from "node:worker_threads";
     import { assert } from "${common}";
-    
+
     const ops = Deno[Deno.internal].core.ops;
-    const obj = ops.op_napi_open("${path}", {}, Buffer, reportError);
+    const obj = ops.op_napi_open("${path}", {}, Buffer.from, reportError);
     assert(obj != null);
     assert(typeof obj === "object");
     parentPort.postMessage("ok");
